#!/usr/bin/python

import urwid

"""
NOTES
-----
This module builds the widget to show the structure information for a table.

"""

def show_table_structure(user_info, tablename):
  #build out the table that shows the table structure information
  #get table info
  table_info = user_info.db_obj.gettableinfo(user_info.db_conn, tablename)

  #building out list of column names
  table_col_names = []
  for x in table_info:
    table_col_names.append(urwid.Text(x[0]))

  #building out list of data types
  table_data_type = []
  for x in table_info:
    table_data_type.append(urwid.Text(x[1]))

  #building out list of character lengths
  table_char_length = []
  for x in table_info:
    if x[2] == None:
      table_char_length.append(urwid.Text(u" "))
    else:
      table_char_length.append(urwid.Text(str(x[2])))

  #building out list of null
  table_null = []
  for x in table_info:
    if x[3] == None:  
      table_null.append(urwid.Text(u" "))
    else:
      table_null.append(urwid.Text(x[3]))

  #building out list of default values
  table_default = []
  for x in table_info:
    if x[4] == None:  
      table_default.append(urwid.Text(u" "))
    else:
      table_default.append(urwid.Text(x[4]))

  #widgets for the table info table
  col_names = urwid.LineBox( urwid.Pile(table_col_names)
    , title="Name", rline=' ', trcorner=u'\u2500', brcorner=u'\u2500')
  datatypes = urwid.LineBox( urwid.Pile(table_data_type)
    , title="Type", rline=' ', trcorner=u'\u2500', brcorner=u'\u2500')
  charlengths = urwid.LineBox( urwid.Pile(table_char_length)
    , title="Length", rline=' ', trcorner=u'\u2500', brcorner=u'\u2500')
  nulls =urwid.LineBox( urwid.Pile(table_null)
    , title="Is Null", rline=' ', trcorner=u'\u2500', brcorner=u'\u2500')
  defaults = urwid.LineBox( urwid.Pile(table_default)
    , title="Default")

  text_1 = urwid.Text(u"Here is the table structure information. If the table does not look aligned, then please make your terminal wider.")

  #main widget for view
  structure_view = urwid.Padding( urwid.Pile( [
    urwid.Divider(),
    text_1,
    urwid.Divider(),
    urwid.Text([u"Structure for table: ", tablename]),
    urwid.Divider(),
    urwid.Columns([
        col_names,
        datatypes,
        charlengths,
        nulls,
        defaults
      ])
    ])
  , left=2, right=2)

  structure_view = urwid.WidgetPlaceholder(structure_view)

  #return the widget created that holds all the structure data
  return structure_view



#!/usr/bin/python

import urwid
import mainview

"""
NOTES
-----
This module builds the widget to allow the user to enter in their own SQL query

This module will also run the sql query and show a success message if it works

"""

class Qinfo:
  def __init__(self):
    self.query_text = None
    self.query_status = None

class Tracking:
  def __init__(self):
    self.start = 0
    self.end = 15

def show_runsql(frame, body, user_info):
  #used to easily insert a blank line widget
  blank = urwid.Divider()
  query_info = Qinfo()

  # reorganizes row list for a column-oriented view, e.g.
  # returns a list of widget lists
  def splitTable(allrows):
    cols = []
    if allrows:
      for i in range(0, len(allrows[0])):
        col = []
        for index, row in enumerate(allrows):
          col.append(urwid.Text(str(row[i])))
        cols.append(col)

    return cols

  #do all of the work to show the select query
  def show_select_results(data, text):
    location = Tracking()
    row_length = len(data)
    col_length = len(data[0])
    widget_lists = splitTable(data)    # get a list of a list of widgets

    #clear out previous query text
    sql_edit.original_widget.set_edit_text(u"")

    #generates tables of 15 starting at a certain row
    def generate_table(start, end):
      columns = []          # empty columns list
      for i in range (0, col_length):    # for each column
        if widget_lists:        # if list not empty
          include_list = []     # list to hold just the widgets to make table out of
          for y in range(start, end):
            include_list.append(widget_lists[i][y])
          mypile = urwid.Pile(include_list) # make a Pile with the list of widgets
        else:
          mypile = urwid.Pile([     # a blank widget to fill up some space
          urwid.Text(u""),
        ])

        # make a linebox with the Pile and the columnname
        if i == col_length - 1:
          mylinebox = urwid.LineBox(mypile)
        else:
          mylinebox = urwid.LineBox((mypile), rline=' ', trcorner=u'\u2500', brcorner=u'\u2500')

        columns.append(mylinebox)     # append the linebox to the list of columns

      return urwid.Columns(columns)
    
    #signal handler for the more button
    def more_btn_press(button):
      if location.end < row_length - 15:
        #can still render a full set of 15 rows
        location.end += 15
        location.start += 15
        table.original_widget = generate_table(location.start, location.end)
        count.set_text([u"Viewing rows ", str(location.start + 1), " - ", str(location.end)])
      elif location.end < row_length:
        #last chuck of data to show
        location.start = location.end
        location.end = row_length
        table.original_widget = generate_table(location.start, location.end)
        count.set_text([u"Viewing rows ", str(location.start + 1), " - ", str(location.end)])

    #signal handler for the less button
    def less_btn_press(button):
      if location.start >= 15:
        location.end = location.start
        location.start = location.end - 15
        table.original_widget = generate_table(location.start, location.end)
        count.set_text([u"Viewing rows ", str(location.start + 1), " - ", str(location.end)])

    #more button to show more results
    #only show if results are greater than a certain amount
    more_btn = urwid.AttrWrap( urwid.Button(u"Next", more_btn_press), 'btnf', 'btn')
    more_btn = urwid.Padding(more_btn, width=8)

    #less button to go back
    less_btn = urwid.AttrWrap( urwid.Button(u"Previous", less_btn_press), 'btnf', 'btn')
    less_btn = urwid.Padding(less_btn, width=12)

    select_text_1 = urwid.Text(["The results from the following SELECT query are below.", "\n\nQUERY: ", text])
    select_text_2 = urwid.Text([u"Total Rows: ", str(row_length)])
    count = urwid.Text([u"Viewing rows ", str(location.start + 1), " - ", str(location.end)])

    if row_length < 15:
      #render just the data available
      table = generate_table(0, row_length)

      #clear out more and less buttons
      more_btn.original_widget = urwid.Text(u"")
      less_btn.original_widget = urwid.Text(u"")
    else:
      table = urwid.WidgetPlaceholder(generate_table(location.start, location.end))

    #build out selection results view
    select_results.original_widget = urwid.Pile([
      select_text_1,
      blank,
      select_text_2,
      count,
      blank,
      table,
      blank,
      urwid.Columns([
        ('fixed', 12, less_btn),
        ('fixed', 3, urwid.Text(u"   ")),
        ('fixed', 8, more_btn)
      ])
    ])

  #signal handler for text input, stores input information from user
  def edit_change_event(self, text):
    query_info.query_text = text

  #signal handler for the run button
  def run_btn_press(button):
    #clear out any previous error messages
    text_error.original_widget = urwid.AttrWrap( urwid.Text(u""), 'body')

    #clear out any previous data results
    select_results.original_widget = urwid.Text(u"")

    if query_info.query_text != None:
      #convert string to all uppercase to search for select
      query_copy = query_info.query_text.upper()

      #identify if query string is a select query
      select = False
      if 'SELECT' in query_copy:
        select = True
      
      #run query
      query_info.query_status = user_info.db_obj.runquery(user_info.db_conn, query_info.query_text, select)

      if query_info.query_status['success'] == True:
        if select:
          if query_info.query_status['data']:
            #query was a select query and has data, show select data
            show_select_results(query_info.query_status['data'], query_info.query_text)
          else:
            text_error.original_widget = urwid.AttrWrap( urwid.Text([u" SELECT query did not return any data", "\n QUERY: ", query_info.query_text]), 'error')
        else:
          #show success message
          frame.footer = urwid.AttrWrap(urwid.Text(u" Query executed successfully"), 'header')
          
          #reload main view. this updates tables list if table was created
          mainview.show_main_view(frame, body, user_info)
      else:
        text_error.original_widget = urwid.AttrWrap( urwid.Text(query_info.query_status['data']), 'error')
    else:
      text_error.original_widget = urwid.AttrWrap( urwid.Text(u" You have enter in a query."), 'error')

  #variables to hold text to show user for login view
  text_1 = urwid.Text(u"Enter a SQL query to run below:")
  text_2 = urwid.Text(u"(The edit box supports multiple lines when you press enter)")

  text_error = urwid.AttrMap( urwid.Text(u""), 'body')

  #setting up the edit input widgets for database name and password
  sql_edit = urwid.Edit(caption="", edit_text="", multiline=True)
  urwid.connect_signal(sql_edit, 'change', edit_change_event)
  sql_edit = urwid.AttrWrap(sql_edit, 'btnf', 'btn')

  #run button
  runsql_btn = urwid.AttrWrap( urwid.Button(u"Run", run_btn_press), 'btnf', 'btn')

  #placeholder for any data results from SELECT query
  select_results = urwid.WidgetPlaceholder( urwid.Text(u""))

  #This is the pile widget that holds all of the main body widgets
  runsql = urwid.WidgetPlaceholder(      
      urwid.Pile([
        urwid.Padding(text_error, left=5, width = 50),
        blank,
        urwid.Padding(text_1, left=5),
        urwid.Padding(text_2, left=5),
        urwid.Padding(sql_edit, left=2, right=2),
        blank,
        urwid.Padding(runsql_btn, left=10, width=11),
        blank,
        urwid.Padding(select_results, left=2)
      ]))

  return runsql



#!/usr/bin/python

import urwid
import mainview

"""
NOTES
-----
This module builds the widget to allow the user to create a table in a database.

"""

class CreateTableInfo:
  def __init__(self):
    self.count = None
    self.table_name = ""
    self.table_fields = None
    self.query_string = ""
    self.atr_name = ""
    self.atr_type = ""
    self.atr_null = False
    self.atr_primarykey = False
    self.atr_unique = False
    self.atr_none = True

def show_db_createtable(frame, body, main_body, user_info):
  #used to easily insert a blank line widget
  blank = urwid.Divider()

  #create class instance to hold table creation input
  table_info = CreateTableInfo()

  #signal handler for the create button
  def create_btn_press(button):
    #store number of fields in table_info
    table_info.table_fields = fields_num.value()

    #check for errors
    if table_info.table_fields == 0 or table_info.table_name == "":
      text_error.original_widget = urwid.AttrWrap( urwid.Text(u"Enter in both a name and number of fields."), 'error')
    elif table_info.table_fields > 20:
      text_error.original_widget = urwid.AttrWrap( urwid.Text(u"The max number of fields is 20"), 'error')
    else:
      #user input was correct, go to next view
      second_createtable(frame, body, main_body, user_info, table_info)  

  #signal handler for text input, stores input information from user
  def edit_change_event(self, text):
    table_info.table_name = text

  #variables to hold text to show user for login view
  text_error = urwid.AttrWrap( urwid.Text(u""), 'body')
  text_1 = urwid.Text(u"Create a table below:")
  text_2 = urwid.Text(u"(the number of fields must be less than 20...)")
  
  #setting up the edit input widgets for database name and password
  table_name_edit = urwid.Edit(u"Table Name: ", "")
  urwid.connect_signal(table_name_edit, 'change', edit_change_event)
  table_name_edit = urwid.AttrWrap(table_name_edit, 'main_sel', 'main_self')

  fields_num = urwid.IntEdit(u"Number of Table Fields: ")
  table_fields_edit = urwid.AttrWrap(fields_num, 'main_sel', 'main_self')

  #create button
  table_nextstep_btn = urwid.AttrWrap( urwid.Button(u"Next Step", create_btn_press), 'btnf', 'btn')

  #This is the pile widget that holds all of the main body widgets
  create_table = urwid.WidgetPlaceholder( urwid.Padding(      
      urwid.Pile([
        text_error,
        blank,
        text_1,
        blank, 
        urwid.Padding( urwid.Pile([table_name_edit, table_fields_edit]), left=5, width=45),
        blank,
        text_2,
        blank,
        urwid.Padding(table_nextstep_btn, left=5, width=13)
      ]), left=5, right=5))

  return create_table


def second_createtable(frame, body, main_body, user_info, table_info):
  blank = urwid.Divider()
  table_info.count = 0 #holds count on what attribute is being edited

  text_1 = urwid.Text([u"Creating Table: ", table_info.table_name])
  edit_num = urwid.Text([u"Now editing attribute number: ", str(table_info.count + 1), ' / ', str(table_info.table_fields)])

  #clear out query string first
  table_info.query_string = ""

  #start creating the query string
  table_info.query_string += 'CREATE TABLE ' + table_info.table_name + ' (\n'

  #error box
  error_box = urwid.AttrMap( urwid.Text(u""), 'main_sel')

  #signal handler for try again button
  def try_again(button):
    second_createtable(frame, body, main_body, user_info, table_info)

  #signal handler for the next attribute button
  def next_atr(button):
    if table_info.atr_name == "" or table_info.atr_type == "":
      error_box.original_widget = urwid.AttrWrap( urwid.Text(u"You must enter a name and type."), 'error')
    else:
      error_box.original_widget = urwid.AttrWrap( urwid.Text(u""), 'main_sel')

      table_info.query_string += table_info.atr_name + ' ' + table_info.atr_type

      if table_info.atr_null == True:
        table_info.query_string += ' NOT NULL'

      if table_info.atr_primarykey == True:
        table_info.query_string += ' PRIMARY KEY'

      if table_info.atr_unique == True:
        table_info.query_string += ' UNIQUE'

      #increment count to reflect new addition of data
      table_info.count += 1
      
      if table_info.count < table_info.table_fields:
        #call function to bring up next form
        next_form()
      else:
        #call function to execute create table query
        create_table()

  #next button
  atr_next_btn = urwid.AttrWrap( urwid.Button(u"Next", next_atr), 'main_sel', 'main_self')
  atr_next_btn = urwid.WidgetPlaceholder(atr_next_btn)

  #signal handler for edit field events
  def edit_change_event(self, text):
    if self.caption == "Name: ":
      table_info.atr_name = text
    elif self.caption == "Type: ":
      table_info.atr_type = text

  #widget for attribute name edit field
  atr_name_edit = urwid.Edit(u"Name: ", "")
  urwid.connect_signal(atr_name_edit, 'change', edit_change_event)
  atr_name_edit = urwid.AttrWrap(atr_name_edit, 'main_sel', 'main_self')

  #widget for type edit field
  atr_type_edit = urwid.Edit(u"Type: ", "")
  urwid.connect_signal(atr_type_edit, 'change', edit_change_event)
  atr_type_edit = urwid.AttrWrap(atr_type_edit, 'main_sel', 'main_self')  

  #signal handler for checkbox
  def checkbox_change(self, state):
    if state == True:
      table_info.atr_null = True
    else:
      table_info.atr_null = False

  #widget for null checkbox
  null_checkbox = urwid.CheckBox(u"Not Null", state=False, on_state_change=checkbox_change)
  null_checkbox = urwid.AttrWrap(null_checkbox, 'main_sel', 'main_self')

  #signal handler for radio buttons
  def radio_change(self, state):
    if self.label == "Primary Key":
      if state == True:
        table_info.atr_primarykey = True
        table_info.atr_unique = False
        table_info.atr_none = False
    elif self.label == "Unique":
      if state == True:
        table_info.atr_primarykey = False
        table_info.atr_unique = True
        table_info.atr_none = False
    elif self.label == "None":
      if state == True:
        table_info.atr_primarykey = False
        table_info.atr_unique = False
        table_info.atr_none = True

  #widgets for radio buttons
  radio_list = []
  primarykey_radio = urwid.AttrWrap( urwid.RadioButton(radio_list, u"Primary Key", False, on_state_change=radio_change), 'main_sel', 'main_self')
  unique_radio = urwid.AttrWrap( urwid.RadioButton(radio_list, u"Unique", False, on_state_change=radio_change), 'main_sel', 'main_self')
  none_radio = urwid.AttrWrap( urwid.RadioButton(radio_list, u"None", True, on_state_change=radio_change), 'main_sel', 'main_self')

  #create button placeholder
  table_create_btn = urwid.WidgetPlaceholder( urwid.Text(u""))

  #signal handler for create button
  def create_table():
    table_info.query_string += '\n);'
    
    #run query
    query_status = user_info.db_obj.runquery(user_info.db_conn, table_info.query_string, False)

    if query_status['success']:
      #query was successful, show success message and change view
      frame.footer = urwid.AttrWrap( urwid.Text(u" Table created successfully"), 'header')

      mainview.show_main_view(frame, body, user_info)
    else: 
      #query failed, show error message
      error_box.original_widget = urwid.AttrWrap( urwid.Text(
        [u"Query Failed. Select 'Try Again' below to re-enter attribute information, or 'Create Table' above to start over.\n\n", query_status['data'], "\nQUERY:  ", table_info.query_string]), 'error')

      #if attributes = 1, next button will still be there
      if table_info.table_fields == 1:
        attribute_box.focus_position = 2  
        atr_next_btn.original_widget = urwid.AttrWrap( urwid.Text(u""), 'main_sel')
           
      #clear out create table button and make it try again button
      table_create_btn.original_widget = urwid.AttrWrap( urwid.Button(u"Try Again", try_again), 'main_sel', 'main_self')

  #controls the looping nature of the repetivie process of entering in data for attributes
  def next_form():
    #clear form
    atr_name_edit.set_edit_text(u"")
    atr_type_edit.set_edit_text(u"")
    null_checkbox.set_state(False)
    none_radio.set_state(True)

    #change focus to name input field
    attribute_box.focus_position = 2

    #change attribute count to show current attribute
    edit_num.set_text([u"Now editing attribute number: ", str(table_info.count + 1), ' / ', str(table_info.table_fields)])

    #keep processing data
    table_info.query_string += ',\n'

    if table_info.count == table_info.table_fields - 1:
      #this is the last attribute to edit
      #remove next button and replace with create button
      atr_next_btn.original_widget = urwid.AttrWrap( urwid.Text(u""), 'main_sel')
      table_create_btn.original_widget = urwid.AttrWrap( urwid.Button(u"Create", next_atr), 'main_sel', 'main_self')

  attribute_box = urwid.Pile([
    error_box,
    blank,
    atr_name_edit,
    blank,
    atr_type_edit,
    blank,
    null_checkbox,
    blank,
    primarykey_radio,
    unique_radio,
    none_radio,
    blank,
    urwid.Padding(atr_next_btn, left=15, width=10)
  ])

  create_attribute = urwid.WidgetPlaceholder( urwid.Padding(
    urwid.Pile([
      text_1,
      blank,
      edit_num,
      blank,
      urwid.LineBox(attribute_box),
      blank,
      urwid.Padding(table_create_btn, left=5, width=10)
    ]), left=5, right=5))

  main_body.original_widget = create_attribute



#!/usr/bin/python

import urwid
import mainview

"""
NOTES
-----
These are the possible colors we can use:
black, dark red, dark green, brown, dark blue, dark magenta, dark cyan, light gray

This is the code to create and run the login view for the program.
If a log in is successful, then the code here will initialize the code that will
create the main view for the program, with a connected DB.

"""


def create_main_view(user_info):
  #used to easily insert a blank line widget
  blank = urwid.Divider()

  #signal handler for the connect button
  def db_connect(button):
    if user_info.mysql == True:
      import mysqlDB as sqlDB	# for MySQL
      user_info.db_obj = sqlDB.MYsql()
    elif user_info.psql == True:
      import psqlDB as sqlDB	# for PostgreSQL
      user_info.db_obj = sqlDB.Psql()

    if (user_info.db_name and user_info.db_uname):
      # connect to the db
      user_info.db_conn = user_info.db_obj.connectdb(user_info.db_name, user_info.db_uname, user_info.db_pw)

      # if connection error returned
      if user_info.db_conn == -1:
        db_error_box.original_widget = urwid.AttrMap( urwid.Text
        (u" Incorrect username, database name, or password.\n Please try again"),
        'error', 'error')

      # if good connection returned
      else:
        #build out and show the main app view on the screen
        mainview.show_main_view(frame, body, user_info)

    else:
      db_error_box.original_widget = urwid.AttrMap( urwid.Text
        (u" Please enter your username and database name"),
        'error', 'error')

  #signal handler for radio buttons, stores input information from user
  def radio_change(self, state):
    if self.label == "MySQL":
      if state == True:
        user_info.mysql = True
        user_info.psql = False
    elif self.label == "PostgreSQL":
      if state == True:
        user_info.psql = True
        user_info.mysql = False

  #signal handler for text input, stores input information from user
  def edit_change_event(self, text):
    if self.caption == u"Database name: ":
      user_info.db_name = text
    elif self.caption == u"Database password: ":
      user_info.db_pw = text
    elif self.caption == u"Username: ":
      user_info.db_uname = text

  #variables to hold text to show user for login view
  text_mainbody_1 = urwid.Text(u"First, please use the below radio buttons to select either a MySQL or PostgreSQL database to connect to.")
  text_mainbody_2 = urwid.Text(u"Now, please enter in the database name and password below in order to connect to the database.")  

  #setting up radio elements for MySQL or PostgreSQL choice
  radio_list = []
  user_info.psql = True		# default to PostgreSQL
  user_info.mysql = False
  mysql_radio = urwid.AttrWrap( urwid.RadioButton(radio_list, u"MySQL", False, on_state_change=radio_change), 'main_sel', 'main_self')
  psql_radio = urwid.AttrWrap( urwid.RadioButton(radio_list, u"PostgreSQL", True, on_state_change=radio_change), 'main_sel', 'main_self')

  #setting up the edit input widgets for database name and password
  db_uname_edit = urwid.Edit(u"Username: ", "")
  urwid.connect_signal(db_uname_edit, 'change', edit_change_event)
  db_uname_edit = urwid.AttrWrap(db_uname_edit, 'main_sel', 'main_self')

  db_name_edit = urwid.Edit(u"Database name: ", "")
  urwid.connect_signal(db_name_edit, 'change', edit_change_event)
  db_name_edit = urwid.AttrWrap(db_name_edit, 'main_sel', 'main_self')

  db_pw_edit = urwid.Edit(u"Database password: ", "", mask=u"*")
  urwid.connect_signal(db_pw_edit, 'change', edit_change_event)
  db_pw_edit = urwid.AttrWrap(db_pw_edit, 'main_sel', 'main_self')

  #connect button
  db_connect_btn = urwid.AttrWrap( urwid.Button(u"Connect", db_connect), 'main_sel', 'main_self')

  #error box
  db_error_box = urwid.AttrMap( urwid.Text(u""), 'main_sel')

  #This is the pile widget that holds all of the main body widgets
  body = urwid.WidgetPlaceholder( urwid.Filler( urwid.Padding(      
    urwid.Pile([
      text_mainbody_1,
      blank, 
      urwid.Padding( urwid.Pile([psql_radio, mysql_radio]), width=14, left=5),
      blank,
      text_mainbody_2,
      blank,
      urwid.Padding( urwid.Pile([db_uname_edit, db_name_edit, db_pw_edit]), left=5, width=45),
      blank,
      urwid.Padding(db_connect_btn, left=5, width=11),
      blank,
      urwid.Padding(db_error_box, left=5, width=50)
    ]), left=5, right=5),
  valign='top', top=3))
  
  #adding color styling to the body widget
  body = urwid.AttrWrap(body, 'bg')

  #Setting up frame
  frame_header = urwid.Padding( urwid.Text(u"Welcome to viewmydb! 'q' exits the pogram. Use the keyboard arrows to navigate."), left=1, right=1)
  frame_header = urwid.AttrWrap(frame_header, 'header')
  frame = urwid.Frame(body=body, header=frame_header)

  #return the frame all set up with body widget
  return frame



#!/usr/bin/python

import urwid

"""
NOTES
-----
This module dynamically builds a table to show all the rows in a
table including column names.

"""

class Tracking:
  def __init__(self):
    self.start = 0
    self.end = 15
  

# reorganizes row list for a column-oriented view, e.g.
# returns a list of widget lists
def splitTable(allrows):
  cols = []
  if allrows:
    # for each column
    for i in range(0, len(allrows[0])):
      col = [] # start a blank column
      # for each row, add to a column
      for index, row in enumerate(allrows):
        col.append(urwid.Text(str(row[i]))) # add a text cell w/ the row's value from the current column
      cols.append(col)  # add the column to the list of columns

  return cols

# creates a widget consisting of a table of all data in the rows
# with column headers
# takes a list of column names and a list of (list)rows

#def showTables(colnames, rowdata):
def showTables(colnames, rowdata, tablefunction, tablebutton, tablename, user_info, frame):
  location = Tracking()
  rows_length = len(rowdata)            # amount of total rows in data
  widget_lists = splitTable(rowdata)    # get a list of a list of widgets

  #generates tables of 15 starting at a certain row
  def generate_table(start, end):
    columns = []					# empty columns list
    for i in range (0, len(colnames)):		# for each column
      if widget_lists:				# if list not empty
        include_list = []     # list to hold just the widgets to make table out of
        for y in range(start, end):
          include_list.append(widget_lists[i][y])
        mypile = urwid.Pile(include_list)	# make a Pile with the list of widgets
      else:
        mypile = urwid.Pile([			# a blank widget to fill up some space
        urwid.Text(u""),
      ])

      # make a linebox with the Pile and the columnname
      if i == len(colnames) - 1:
        mylinebox = (urwid.LineBox((mypile), title=colnames[i]))
      else:
        mylinebox = (urwid.LineBox((mypile), title=colnames[i], rline=' ', trcorner=u'\u2500', brcorner=u'\u2500'))

      columns.append(mylinebox)			# append the linebox to the list of columns

# builds the Delete buttons in the first column
################################################################################################################
    prepile = []
    for j in range (start, end):
      prepile.append(urwid.Padding(urwid.Button((u"Delete Row"), row_delete_btn_press, rowdata[j]), width=14))

    newpile = urwid.Pile(prepile)
    newlinebox = urwid.LineBox(newpile, title="Delete", rline=' ', trcorner=u'\u2500', brcorner=u'\u2500')
    columns.insert(0, newlinebox)
################################################################################################################

    return urwid.Columns(columns)

# handles presses for the delete buttons
####################################################################
  def row_delete_btn_press(button, row):
    # delete the row
    query = "DELETE FROM " + tablename + " WHERE "
    for k, name in enumerate(colnames):
      if str(row[k]) != "None":
        if k != 0:
          query += " AND "
        query += name
        query += "='"
        query += (str(row[k]) + "'")
    query += ";"
    query_status = user_info.db_obj.runquery(user_info.db_conn, query, 0)
    
    if query_status['success']:
      #show success message
      frame.footer = urwid.AttrWrap( urwid.Text(u" Row deleted successfully"), 'header')
      # go back to the table view
      tablefunction(tablebutton, tablename)
    else:
      #show error message
      error_msg.original_widget = urwid.AttrWrap( urwid.Text(query_status['data']), 'error')
####################################################################

  #signal handler for the more button
  def more_btn_press(button):
    if location.end < rows_length - 15:
      #can still render a full set of 15 rows
      location.end += 15
      location.start += 15
      table.original_widget = generate_table(location.start, location.end)
      count.set_text([u"Viewing rows ", str(location.start + 1), " - ", str(location.end)])
    elif location.end < rows_length:
      #last chuck of data to show
      location.start = location.end
      location.end = rows_length
      table.original_widget = generate_table(location.start, location.end)
      count.set_text([u"Viewing rows ", str(location.start + 1), " - ", str(location.end)])

  #signal handler for the less button
  def less_btn_press(button):
    if location.start >= 15:
      location.end = location.start
      location.start = location.end - 15
      table.original_widget = generate_table(location.start, location.end)
      count.set_text([u"Viewing rows ", str(location.start + 1), " - ", str(location.end)])

  #more button to show more results
  #only show if results are greater than a certain amount
  more_btn = urwid.AttrWrap( urwid.Button(u"Next", more_btn_press), 'btnf', 'btn')
  more_btn = urwid.Padding(more_btn, width=8)

  #less button to go back
  less_btn = urwid.AttrWrap( urwid.Button(u"Previous", less_btn_press), 'btnf', 'btn')
  less_btn = urwid.Padding(less_btn, width=12)

  text_1 = urwid.Text(u"View table data below. If the columns or column names are not rendering correctly, then make your terminal wider. The table displays data in pages of 15 rows, click the more or less button to cycle through the data.")

  text_2 = urwid.Text([u"Total Rows: ", str(rows_length)])

  count = urwid.Text([u"Viewing rows ", str(location.start + 1), " - ", str(location.end)])

  error_msg = urwid.AttrWrap(urwid.Text(u""), 'body')

  if rows_length < 15:
    #render just the data available
    table = generate_table(0, rows_length)

    #clear out more and less buttons
    more_btn.original_widget = urwid.Text(u"")
    less_btn.original_widget = urwid.Text(u"")
  else:
    table = urwid.WidgetPlaceholder(generate_table(location.start, location.end))

  structure_view = urwid.WidgetPlaceholder( urwid.Padding( urwid.Pile([
      urwid.Divider(),
      text_1,
      urwid.Divider(),
      text_2,
      count,
      urwid.Divider(),
      error_msg,
      table,
      urwid.Divider(),
      urwid.Columns([
        ('fixed', 12, less_btn),
        ('fixed', 3, urwid.Text(u"   ")),
        ('fixed', 8, more_btn)
      ])
    ]), left=2, right=2))

  #return the widget created that holds all the structure data
  return structure_view




#!/usr/bin/python

import urwid
import mainview

"""
NOTES
-----
This module builds a widget with a warning message for dropping a table

"""

def show_table_drop(frame, body, user_info, tablefunction, tablebutton):
  blank = urwid.Divider()

  def yes_drop(button):
    #execute the drop query
    status = user_info.db_obj.drop_table(user_info.db_conn, tablebutton.get_label())

    if status == 1:
      #show success message in frame footer
      frame.footer = urwid.AttrWrap( urwid.Text([u" Table ",tablebutton.get_label() , " successfully dropped"]), 'header')

      #redirect to main view
      mainview.show_main_view(frame, body, user_info)
    else:
      text_error.original_widget = urwid.AttrWrap( urwid.Text(status), 'error')

  def no_drop(button):
    #redirect to table view
    tablefunction(tablebutton)

  text_1 = urwid.Text(u"WARNING!!!!!\nDropping a table permanently deletes this table and all its data from the database.")
  text_2 = urwid.Text([u"Do you wish to proceed with dropping table ", tablebutton.get_label(), "?"])
  text_error = urwid.AttrWrap( urwid.Text(u""), 'body')

  table_drop_yes = urwid.AttrWrap( urwid.Button(u"Yes", yes_drop), 'btnf', 'btn')
  table_drop_no = urwid.AttrWrap( urwid.Button(u"No", no_drop), 'btnf', 'btn')

  drop_table = urwid.WidgetPlaceholder( urwid.Padding(
    urwid.Pile([
      blank,
      text_error,
      text_1,
      text_2,
      blank,
      urwid.Columns([
        ('fixed', 6, table_drop_no),
        ('fixed', 5, urwid.Text(u"     ")),
        ('fixed', 7, table_drop_yes)
      ]),
    ]), left=5, right=5))

  return drop_table



#!/usr/bin/python

import urwid

"""
NOTES
-----
This module builds the widget to show the structure information for a database.

"""

def show_db_structure(user_info):
  #build out the table that shows the db structure stats
  #get db info
  db_info = user_info.db_obj.getdbinfo(user_info.db_conn, user_info.db_name)

  #get table names and size
  db_tableinfo = user_info.db_obj.getdb_tableinfo(user_info.db_conn)

  #building out list of table name text widgets
  table_names_list = []
  for x in db_tableinfo:
    table_names_list.append(urwid.Text(x[0]))
  
  #building out list of table size text widgets
  table_sizes_list = []
  for x in db_tableinfo:
    table_sizes_list.append(urwid.Text(x[1]))


  #widgets for the database info table
  db_name_col = urwid.LineBox( 
      urwid.Text(db_info[0])
    , title="DB Name", rline=' ', trcorner=u'\u2500', brcorner=u'\u2500')
  encoding_col = urwid.LineBox( 
      urwid.Text(db_info[1])
    , title="Encoding", rline=' ', trcorner=u'\u2500', brcorner=u'\u2500')
  collation_col = urwid.LineBox( 
      urwid.Text(db_info[2])
    , title="Collation")

  #widgets for the database tables info table
  table_names = urwid.LineBox( urwid.Pile(table_names_list)
    , title="Name", rline=' ', trcorner=u'\u2500', brcorner=u'\u2500')
  table_sizes = urwid.LineBox( urwid.Pile(table_sizes_list)
    , title="Size")

  #main widget for view
  structure_view = urwid.Padding( urwid.Pile( [
    urwid.Text(u"Database Info:"),
    urwid.Columns([
        db_name_col,
        encoding_col,
        collation_col
      ]),
    urwid.Divider(),
    urwid.Text(u"DB Tables:"),
    urwid.Columns([
        table_names,
        table_sizes
      ])
    ])
  , left=2, right=2)

  structure_view = urwid.WidgetPlaceholder(structure_view)

  #return the widget created that holds all the structure data
  return structure_view



#!/usr/bin/python

import psycopg2

"""
NOTES
-----

This file contains the functions for the PostgreSQL implementation part of the code.

"""

class Psql:
  # takes database name, username, password
  # returns psycopg2 cursor object on success, -1 on failure
  def connectdb(self, db, un, pw):
    try:
      conn = psycopg2.connect(database=db, user=un, password=pw)
      #conn = psycopg2.connect(database="test1", user="postgres", password="cs419db")
      return conn
    except psycopg2.OperationalError:
      return -1

  # takes psycopg2 cursor object
  # returns list of table names on success, -1 on failure
  def gettables(self, conn):
    cur = conn.cursor()
    try:
      cur.execute("""SELECT DISTINCT table_name FROM information_schema.tables 
      WHERE table_schema='public' AND table_type='BASE TABLE' ORDER BY table_name;""")
      data = cur.fetchall()
      #print data
      names = []
      for table in data:
        names.append(table[0])
      cur.close()
      return names
    except:
      cur.close()
      return -1

  # takes psycopg2 cursor object
  # returns list of row data on success, -1 on failure
  def allrows(self, conn, name):
    cur = conn.cursor()
    SQL = "SELECT * FROM " + name + ";"
    try:
      cur.execute(SQL)
      data = cur.fetchall()
      rows = []
      for index, row in enumerate(data):
        newrow = []
#        newrow.append(index)
        for element in row:
          newrow.append(element)
        rows.append(newrow)
      cur.close()
      return rows
    except:
      cur.close()
      return -1

  # takes psycopg2 cursor object
  # returns list of all column names on success, -1 on failure
  def getcolnames(self, conn, table):
    cur = conn.cursor()
    try:
      SQL = "SELECT column_name FROM information_schema.columns WHERE table_name = " + "'" + table + "';"
      cur.execute(SQL)
      data = cur.fetchall()
      columns = []
#      columns.append("Index")
      for column in data:
        columns.append(column[0])
      cur.close()
      return columns
    except:
      cur.close()
      return -1

  # takes psycopg2 cursor object and text string of query to run
  # returns 1 on success and error message on failure
  def runquery(self, conn, text, select):
    cur = conn.cursor()
    try:
      cur.execute(text)
      conn.commit()
      if select:
        data = cur.fetchall()
      else:
        data = ""
      cur.close()
      return {'success':True, 'data':data}
    except psycopg2.Error as e:
      conn.commit()
      cur.close()
      return {'success':False, 'data':e.pgerror}

  def getdbinfo(self, conn, name):
    cur = conn.cursor()
    try:
      cur.execute("""SELECT d.datname AS "Name",
          pg_catalog.pg_encoding_to_char(d.encoding) AS "Encoding",
          d.datcollate AS "Collate"
        FROM pg_catalog.pg_database d
        WHERE datname=(%s);""", (name,))
      data = cur.fetchall()
      cur.close()
      return data[0]
    except:
      cur.close()
      return -1

  def getdb_tableinfo(self, conn):
    cur = conn.cursor()
    try:
      cur.execute("""SELECT relname AS "Table",
          pg_size_pretty(pg_total_relation_size(relid)) AS "Size"
        FROM pg_catalog.pg_statio_user_tables
        ORDER BY pg_total_relation_size(relid) DESC;""")
      data = cur.fetchall()
      cur.close()
      return data
    except:
      cur.close()
      return -1

  def gettableinfo(self, conn, tablename):
    cur = conn.cursor()
    try:
      cur.execute("""SELECT column_name, data_type, character_maximum_length, is_nullable, column_default
        FROM INFORMATION_SCHEMA.COLUMNS
        WHERE table_name=(%s);""", (tablename,))
      data = cur.fetchall()
      cur.close()
      return data
    except:
      cur.close()
      return -1

  def truncate_table(self, conn, tablename):
    cur = conn.cursor()
    try:
      SQL = "TRUNCATE " + str(tablename)
      cur.execute(SQL)
      conn.commit()
      cur.close()
      return 1
    except psycopg2.Error as e:
      conn.commit()
      cur.close()
      return e.pgerror

  def drop_table(self, conn, tablename):
    cur = conn.cursor()
    try:
        SQL = "DROP TABLE " + str(tablename)
        cur.execute(SQL)
        conn.commit()
        cur.close()
        return 1
    except psycopg2.Error as e:
      conn.commit()
      cur.close()
      return e.pgerror

  def rename_table(self, conn, tablename, newname):
    cur = conn.cursor()
    try:
        SQL = "ALTER TABLE " + str(tablename) + " RENAME TO " + str(newname)
        cur.execute(SQL)
        conn.commit()
        cur.close()
        return 1
    except psycopg2.Error as e:
      conn.commit()
      cur.close()
      return e.pgerror



from setuptools import setup

def readme():
  with open('README.md') as f:
    return f.read()

setup(
  name='viewmydb',
  version='0.1.4',
  description='Python terminal GUI for MySQL and PostgreSQL DBs.',
  classifiers=[
    'Development Status :: 4 - Beta',
    'Environment :: Console :: Curses',
    'Programming Language :: Python :: 2.7',
    'Intended Audience :: Information Technology',
    'Natural Language :: English',
    'License :: OSI Approved :: MIT License',
    'Operating System :: Unix'
  ],
  keywords=['mysql', 'postgresql', 'urwid', 'ncurses', 'curses'],
  url='http://github.com/OSU-CS419/viewmydb',
  author="reformingpanda, davidtadams",
  author_email="adams.t.david@gmail.com",
  license='MIT',
  scripts=['bin/viewmydb'],
  packages=['viewmydb'],
  install_requires=[
    'urwid',
    'psycopg2',
    'MySQL-python'
  ],
  test_suite='nose.collector',
  tests_require=['nose'],
  include_package_data=True,
  zip_safe=False
)


#!/usr/bin/python

import urwid
import mainview

"""
NOTES
-----
This module builds the widget to show the edit view for the table.

"""

class EditTableInfo:
  def __init__(self):
    self.rename_name = ""
    self.add_field = ""
    self.add_query_string = ""
    self.add_current = 0

def show_table_edit(frame, body, main_body, user_info, tablename):
  blank = urwid.Divider()

  edit_info = EditTableInfo()

  def rename_btn_press(button):
    #make sure new name has been entered in
    if edit_info.rename_name == "":
      #show error for blank entry
      text_error.original_widget = urwid.AttrWrap( urwid.Text(u"Enter in a table name."), 'error')
    else:
      #sql function to rename table
      status = user_info.db_obj.rename_table(user_info.db_conn, tablename, edit_info.rename_name)

      if status == 1:
        #show success message in frame footer
        frame.footer = urwid.AttrWrap( urwid.Text([u" Table ", tablename, " renamed to ", edit_info.rename_name]), 'header')

        #redirect to main view to reload new table name
        mainview.show_main_view(frame, body, user_info)
      else:
        #show error message
        text_error.original_widget = urwid.AttrWrap( urwid.Text(status), 'error')

    
  #signal handler for rename table input field
  def edit_change_event(self, text):
    edit_info.rename_name = text

  text_1 = urwid.Text([u"Rename table: ", tablename])
  text_error = urwid.AttrWrap( urwid.Text(u""), 'body')

  #rename edit input field
  table_rename_edit = urwid.Edit(u"New name: ", "")
  urwid.connect_signal(table_rename_edit, 'change', edit_change_event)
  table_rename_edit = urwid.AttrWrap(table_rename_edit, 'main_sel', 'main_self')

  #rename button
  table_rename_btn = urwid.AttrWrap( urwid.Button(u"Rename", rename_btn_press), 'btnf', 'btn')

  #---------------------------------------------------------------------------------
  #setting up the add a row view
  col_names = user_info.db_obj.getcolnames(user_info.db_conn, tablename)
  col_length = len(col_names)

  #clear out and start creating query string
  edit_info.add_query_string = ""
  edit_info.add_query_string += 'INSERT INTO ' + tablename + ' VALUES ('
  
  add_error = urwid.AttrWrap( urwid.Text(u""), 'main_sel')
  add_text_1 = urwid.Text(u"Use the below form to add a row of data to the table.\nIf you leave the entry blank, it will default to NULL.\nIf you have a table with an auto-incrementing column, you can type 'DEFAULT' and it will resort to the default value.\nNote that default will also work for other columns with default values set up.")

  field_text = urwid.Text(["Now adding data for field: ", str(edit_info.add_current + 1), " / ", str(col_length)])

  def edit_change_event(self, text):
    edit_info.add_field = text

  def try_again(button):
    main_body.original_widget = show_table_edit(frame, body, main_body, user_info, tablename)

  def next_field(button):
    frame.footer = urwid.AttrWrap( urwid.Text(""), 'body')

    field_copy = edit_info.add_field.upper()

    if edit_info.add_field == "":
      edit_info.add_query_string += "NULL"
    elif edit_info.add_field.isdigit() or field_copy == "DEFAULT":
      edit_info.add_query_string += edit_info.add_field
    else:
      edit_info.add_query_string += "'" + edit_info.add_field + "'"
    
    edit_info.add_current += 1

    if edit_info.add_current < col_length:
      #not done, need to load another form
      next_form()
    else:
      #input is done, can run query
      add_row()

  field_input_text = urwid.Edit(["Field->", col_names[edit_info.add_current], ": "])
  urwid.connect_signal(field_input_text, 'change', edit_change_event)
  field_input = urwid.AttrWrap(field_input_text, 'main_sel', 'main_self')

  #check to see if table only has one field, then next button becomes add button
  if edit_info.add_current < col_length:
    #next button
    next_field_btn = urwid.AttrWrap( urwid.Button(u"Next", next_field), 'btnf','btn')
  else:      
    #next button
    next_field_btn = urwid.AttrWrap( urwid.Button(u"Add", next_field), 'btnf','btn')

  next_field_btn = urwid.WidgetPlaceholder(next_field_btn)

  def add_row():
    edit_info.add_query_string += ');'

    #run insert query
    query_status = user_info.db_obj.runquery(user_info.db_conn, edit_info.add_query_string, False)

    if query_status['success']:
      #query was successful, show success message and change view
      frame.footer = urwid.AttrWrap( urwid.Text(u" Row added successfully"), 'header')      
      
      #redirect to show this page again
      main_body.original_widget = show_table_edit(frame, body, main_body, user_info, tablename)
    else:
      #query failed, show error message
      add_error.original_widget = urwid.AttrWrap( urwid.Text(["Query Failed.\nQUERY: ", edit_info.add_query_string, "\n\n", query_status['data']]), 'error')

      #show try again button
      next_field_btn.original_widget = urwid.AttrWrap( urwid.Button("Try Again", try_again), 'btnf', 'btn')
    
  def next_form():
    #clear form
    field_input_text.set_edit_text(u"")

    #show current number to edit
    field_text.set_text(["Now adding data for field: ", str(edit_info.add_current + 1), " / ", str(col_length)])

    #show current field name
    field_input_text.set_caption(["Field->", col_names[edit_info.add_current], ": "])

    #keep processing data
    edit_info.add_query_string += ', '

    if edit_info.add_current == col_length - 1:
      #this is the last attribute to add, replace next button with add button
      next_field_btn.original_widget = urwid.AttrWrap( urwid.Button("Add", next_field), 'btnf', 'btn')

  #----------------------------------------------------------------------------

  add_box = urwid.Pile([
    add_error,
    blank,
    field_text,
    blank,
    urwid.Padding(field_input, right=5),
    blank,
    urwid.Padding(next_field_btn, left=5, width=13)
  ])

  edit_body = urwid.WidgetPlaceholder( urwid.Padding(
    urwid.Pile([
      blank,
      text_error,
      text_1,
      table_rename_edit,
      blank,
      urwid.Padding(table_rename_btn, left=5, width=10),
      blank,
      blank,
      add_text_1,
      blank,
      urwid.LineBox(add_box)
    ]), left=5, right=5))

  return edit_body



#!/usr/bin/python

import urwid
import DBstructure
import DBcreatetable
import runsql
import TableStructure
import TableBrowse
import TableEdit
import TableDrop
import TableTruncate


"""
NOTES
-----
These are the possible colors we can use:
black, dark red, dark green, brown, dark blue, dark magenta, dark cyan, light gray

This is the code that sets up and displays the main dashboard view for the program

"""


def show_main_view(frame, body, user_info):
  blank = urwid.Divider()

  text_instructions = (u"This program allows you to connect to a PostgreSQL or MySQL database and then perform operations on that database. The program is written in python and is an ncurses based command line tool. Anything within < > brackets is a selectable button. If the tables do not look right, please make the console window wider.")

  #--------------------------------------------------------------------
  #This creates the widget for the top most bar

  #Signal Handler for the Run SQL button
  def run_sql(button):
    #run the code that generates the run sql input view
    main_body.original_widget = runsql.show_runsql(frame, body, user_info)

  #Creating the widget that holds the top bar information
  selected = urwid.Text([u" Selected Database: ", user_info.db_name])
  top_bar = urwid.Columns([
    urwid.AttrWrap( selected, 'topmenu'),
    ('fixed', 15, urwid.AttrWrap( urwid.Padding( urwid.Button(
          u"Run SQL", run_sql), 
        right=4), 
      'btnf', 'btn'))
  ])
  #END~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~


  #--------------------------------------------------------------------
  #This creates the left column widget
  text_leftcol1_1 = (u"Database:")
  text_leftcol2_1 = (u"Tables:")

  #signal handler for left column database button
  def leftcol_btn_press_db(button):
    # update the list of tables and associated buttons
    db_tables = user_info.db_obj.gettables(user_info.db_conn)
    table_buttons = urwid.Pile(
      [urwid.AttrWrap( urwid.Button(txt, leftcol_btn_press_table), 'btn', 'btnf') for txt in db_tables ])
    left_column.original_widget = urwid.Padding (urwid.Pile([
          blank,
          urwid.Text(text_leftcol1_1, align='center'),
          db_button,
          urwid.Divider("="),
          blank,
          urwid.Text(text_leftcol2_1, align='center'),
          table_buttons,
          blank
        ])
      , left=1, right=2)

    # update the top menu button selection
    secondary_top.original_widget = urwid.AttrWrap(urwid.Padding(blank), 'bg')   
    main_top.original_widget = urwid.Columns([
      ('fixed', 13, db_structure_button),
      ('fixed', 3, urwid.Text(u"  ")),
      ('fixed', 16, db_createtable_button)
    ])

    main_padding.original_widget = urwid.Pile([
      #tool bar menu
      urwid.AttrWrap( urwid.Divider("-"), 'topmenu'),
      urwid.AttrWrap( main_top, 'topmenu'),
      urwid.AttrWrap( urwid.Divider("-"), 'topmenu'),
      blank,
      blank,
      #main body
      main_body
    ])

    main_body.original_widget = DBstructure.show_db_structure(user_info)
    selected.set_text([u" Selected Database: ", button.get_label()])

  #signal handler for left column table buttons
  def leftcol_btn_press_table(button):
    tablename = button.get_label()
    truncate_text = "Truncate '" + tablename + "'"
    drop_text = "Drop '" + tablename + "'"
    table_truncate_btn = urwid.AttrWrap( urwid.Button(truncate_text, btn_press_table_truncate, button), 'btnf', 'btn')
    table_drop_btn = urwid.AttrWrap( urwid.Button(drop_text, btn_press_table_drop, button), 'btnf', 'btn')
    secondary_top.original_widget = urwid.AttrWrap(urwid.Padding( urwid.Columns([
      ('fixed', (len(truncate_text) + 4), table_truncate_btn),
      ('fixed', 3, urwid.Text(u"  ")),
      ('fixed', (len(drop_text) + 4), table_drop_btn)
    ]), left=2, right=2), 'topmenu')

    db_table_structure_btn = urwid.AttrWrap( urwid.Button(u"Structure", leftcol_btn_press_table_structure, button.get_label()), 'btnf', 'btn')
    db_table_browse_btn = urwid.AttrWrap( urwid.Button(u"Browse", leftcol_btn_press_table_browse, button.get_label()), 'btnf', 'btn')
    db_table_edit_btn = urwid.AttrWrap( urwid.Button(u"Edit", leftcol_btn_press_table_edit, button.get_label()), 'btnf', 'btn')
    main_top.original_widget = urwid.Columns([
      ('fixed', 13, db_table_structure_btn),
      ('fixed', 3, urwid.Text(u"   ")),
      ('fixed', 8, db_table_edit_btn),
      ('fixed', 3, urwid.Text(u"  ")),
      ('fixed', 10, db_table_browse_btn),
    ])

    main_padding.original_widget = urwid.Pile([
      #tool bar menu
      urwid.AttrWrap( urwid.Divider("-"), 'topmenu'),
      urwid.AttrWrap( main_top, 'topmenu'),
      urwid.AttrWrap( urwid.Divider("-"), 'topmenu'),
      urwid.AttrWrap( secondary_top, 'topmenu'),
      urwid.AttrWrap( urwid.Divider("-"), 'topmenu'),
      #main body
      main_body
    ])

    main_body.original_widget = TableStructure.show_table_structure(user_info, tablename)
    selected.set_text([u" Selected Table: ", button.get_label()])

  #signal handler for the table 'structure' button
  def leftcol_btn_press_table_structure(button, tablename):
    main_body.original_widget = TableStructure.show_table_structure(user_info, tablename)

  #signal handler for the table 'browse' button
  def leftcol_btn_press_table_browse(button, tablename):
    main_body.original_widget = TableBrowse.showTables(user_info.db_obj.getcolnames(user_info.db_conn, tablename), user_info.db_obj.allrows(user_info.db_conn, tablename), leftcol_btn_press_table_browse, button, tablename, user_info, frame)

  # signal handler for the table 'edit' button
  def leftcol_btn_press_table_edit(button, tablename):
    main_body.original_widget = TableEdit.show_table_edit(frame, body, main_body, user_info, tablename)

  #signal handler for table 'truncate' button
  def btn_press_table_truncate(button, tablebutton):
    main_body.original_widget = TableTruncate.show_table_trunc(frame, user_info, leftcol_btn_press_table, tablebutton)

  #signal handler for table 'drop' button
  def btn_press_table_drop(button, tablebutton):
    main_body.original_widget = TableDrop.show_table_drop(frame, body, user_info,leftcol_btn_press_table, tablebutton)

  #store database name that user is connected to
  db_name = user_info.db_name

  #store the table names
  db_tables = user_info.db_obj.gettables(user_info.db_conn)

  #create the table button widgets
  table_buttons = urwid.Pile(
    [urwid.AttrWrap( urwid.Button(txt, leftcol_btn_press_table), 'btn', 'btnf') for txt in db_tables ])

  #creating variable for database button widget
  db_button = urwid.AttrWrap( urwid.Button(db_name, leftcol_btn_press_db), 'btn', 'btnf')

  #left column widget variable build
  left_column = urwid.AttrWrap( urwid.Padding (urwid.Pile([
          blank,
          urwid.Text(text_leftcol1_1, align='center'),
          db_button,
          urwid.Divider("="),
          blank,
          urwid.Text(text_leftcol2_1, align='center'),
          table_buttons,
          blank
        ])
      , left=1, right=2)
    , 'leftside')
  #END~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~


  #--------------------------------------------------------------------
  #This will be the tool bar top widget of the main body
  def db_structure_btn_press(button):
    #replace main body with structure data table
    main_body.original_widget = DBstructure.show_db_structure(user_info)

  def db_createtable_btn_press(button):
    #create a table in the database, if error message, show error
    main_body.original_widget = DBcreatetable.show_db_createtable(frame, body, main_body, user_info)

  text_maintop = u" This is the top section of the main body"
  db_structure_button = urwid.AttrWrap( urwid.Button(u"Structure", db_structure_btn_press), 'btnf', 'btn')
  db_createtable_button = urwid.AttrWrap( urwid.Button(u"Create Table", db_createtable_btn_press), 'btnf', 'btn')

  main_top = urwid.Padding( urwid.Columns([
      ('fixed', 13, db_structure_button),
      ('fixed', 3, urwid.Text(u"  ")),
      ('fixed', 16, db_createtable_button)
    ])
  , left=2, right=2)

  secondary_top = urwid.Padding(blank)
  #END~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~


  #--------------------------------------------------------------------
  #This will be the main body widget
  #default view upon first render of screen is database-show structure
  main_body = DBstructure.show_db_structure(user_info)
  #END~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~


  #--------------------------------------------------------------------
  #This is creating the listbox that makes up the body of the main frame

  main_padding = urwid.Padding( urwid.Pile([
    #tool bar menu
    urwid.AttrWrap( urwid.Divider("-"), 'topmenu'),
    urwid.AttrWrap( main_top, 'topmenu'),
    urwid.AttrWrap( urwid.Divider("-"), 'topmenu'),
    blank,
    blank,
    #main body
    main_body
  ]), left=0, right=0)

  #This listbox is the main body of the UI
  listbox_content = [
    blank,
    #instructions section
    urwid.Padding(urwid.Text(text_instructions), left=2, right=2),
    blank,
    urwid.AttrWrap( urwid.Divider(), 'topmenu'),
    #top most bar
    top_bar,
    urwid.AttrWrap( urwid.Divider(), 'topmenu'),
    #the 2 columns holding the left and right side of main body
    urwid.Columns([
      #left column menu
      ('fixed', 17, left_column),
      #right column tool bar and main body
      urwid.AttrWrap(main_padding, 'body')
    ])
  ]
  #END~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  
  #this is the widget that acts as the body of the frame and is a ListBox
  listbox = urwid.ListBox(urwid.SimpleListWalker(listbox_content))
  listbox.set_focus(6)
  listbox = urwid.AttrWrap(listbox, 'bg')

  #this substitutes in the old body for this new listbox body
  body.original_widget = listbox



#!/usr/bin/python

import MySQLdb
from warnings import filterwarnings
filterwarnings('ignore', category= MySQLdb.Warning)

"""
NOTES
-----

This file contains the functions for the mysql implementation part of the code.

"""

class MYsql:
  # takes database name, username, password
  # returns cursor object on success, -1 on failure
  def connectdb(self, name, user, pw):
    try:
      conn = MySQLdb.connect(host="localhost",user=user, passwd=pw, db=name)
      conn.ourdbname = name	# for some reason, it needs the db name for the gettables query
      return conn
    except:
      return -1

  # takes cursor object
  # returns list of table names on success, -1 on failure
  def gettables(self, conn):
    cur = conn.cursor()
    try:
      SQL ="""SELECT DISTINCT TABLE_NAME FROM INFORMATION_SCHEMA.TABLES WHERE TABLE_TYPE = 
             'BASE TABLE' AND TABLE_SCHEMA = '""" + conn.ourdbname +  """';"""
      cur.execute(SQL)
      data = cur.fetchall()
      names = []
      for table in data:
        names.append(table[0])
      cur.close()
      return names
    except:
      cur.close()
      return -1

  # takes cursor object
  # returns list of row data on success, -1 on failure
  def allrows(self, conn, name):
    cur = conn.cursor()
    SQL = "SELECT * FROM " + name + ";"
    try:
      cur.execute(SQL)
      data = cur.fetchall()
      rows = []
      for index, row in enumerate(data):
        newrow = []
#        newrow.append(index)
        for element in row:
          newrow.append(element)
        rows.append(newrow)
      cur.close()
      return rows
    except:
      cur.close()
      return -1

  # takes cursor object
  # returns list of all column names on success, -1 on failure
  def getcolnames(self, conn, table):
    cur = conn.cursor()
    try:
      SQL = """SELECT DISTINCT COLUMN_NAME FROM INFORMATION_SCHEMA.COLUMNS WHERE TABLE_NAME='""" + table + """';"""
      cur.execute(SQL)
      data = cur.fetchall()
      columns = []
#      columns.append("Index")
      for column in data:
        columns.append(column[0])
      cur.close()
      return columns
    except:
      cur.close()
      return -1

  # takes cursor object and text string of query to run
  # returns 1 on success and error message on failure
  def runquery(self, conn, text, select):
    cur = conn.cursor()
    try:
      cur.execute(text)
      conn.commit()
      if select:
        data = cur.fetchall()
      else:
        data = ""
      cur.close()
      return {'success':True, 'data':data}
    except MySQLdb.Error as e:
      conn.commit()
      cur.close()
      return {'success':False, 'data':e.args[1]}

  # gets information about the database
  def getdbinfo(self, conn, name):
    cur = conn.cursor()
    try:
      cur.execute("""SELECT schema_name AS "Name",
     default_character_set_name AS "Encoding",
     DEFAULT_COLLATION_NAME AS "Collate"
     FROM information_schema.SCHEMATA WHERE
     schema_name = (%s);""", (name,))
      data = cur.fetchall()
      cur.close()
      return data[0]
    except:
      cur.close()
      return -1

  # gets information about a table
  def getdb_tableinfo(self, conn):
    cur = conn.cursor()
    name = conn.ourdbname
    try:
      cur.execute("""SELECT table_name AS "Table", 
      round(((data_length + index_length) / 1024)) AS "Size" 
      FROM information_schema.TABLES 
      WHERE table_schema = (%s)
      ORDER BY (data_length + index_length) DESC;""", (name,))
      data = cur.fetchall()
      cur.close()

      # put it in a list instead of a tuple
      info = []
      row = []
      for table in data:
        # cast as a string
        size = str(table[1]) + " kB"
        name = str(table[0])
        row.append(name)	# append the name
        row.append(size)	# append the size
        info.append(row)	# append that row
        row = []		# empty the row

      return info
    except:
      cur.close()
      return -1
  
  # gets info about the table
  def gettableinfo(self, conn, tablename):
    cur = conn.cursor()
    try:
      cur.execute("""SELECT column_name, data_type, character_maximum_length, is_nullable, column_default
        FROM INFORMATION_SCHEMA.COLUMNS
        WHERE table_name=(%s);""", (tablename,))
      data = cur.fetchall()
      cur.close()
      return data
    except:
      cur.close()
      return -1

  # truncates a table
  def truncate_table(self, conn, tablename):
    cur = conn.cursor()
    try:
      SQL = "TRUNCATE " + str(tablename)
      cur.execute(SQL)
      conn.commit()
      cur.close()
      return 1
    except MySQLdb.Error as e:
      conn.commit()
      cur.close()
      return e.pgerror

  # drops a table
  def drop_table(self, conn, tablename):
    cur = conn.cursor()
    try:
        SQL = "DROP TABLE " + str(tablename)
        cur.execute(SQL)
        conn.commit()
        cur.close()
        return 1
    except MySQLdb.Error as e:
      conn.commit()
      cur.close()
      return e.pgerror

  # renames a table
  def rename_table(self, conn, tablename, newname):
    cur = conn.cursor()
    try:
        SQL = "ALTER TABLE " + str(tablename) + " RENAME TO " + str(newname)
        cur.execute(SQL)
        conn.commit()
        cur.close()
        return 1
    except MySQLdb.Error as e:
      conn.commit()
      cur.close()
      return e.pgerror



#!/usr/bin/python

import urwid
import urwid.curses_display
import loginview
import warnings
warnings.filterwarnings("ignore")

"""
NOTES
-----
These are the possible colors we can use:
black, dark red, dark green, brown, dark blue, dark magenta, dark cyan, light gray

"""

#This class is used to store all of the data pertaining to the database connection
#It is used in a similar manner to a C struct
class UserDBInfo:
  def __init__(self):
    self.db_uname = ""
    self.db_name = ""
    self.db_pw = ""
    self.db_conn = None
    self.db_obj = None
    self.psql = False
    self.mysql = False

#This is the color palette used for the program. the format is "name, foreground, background"
palette = [
  ('header', 'light gray', 'dark red'),
  ('topmenu', 'light gray', 'dark blue'),
  ('leftside', 'light gray', 'dark cyan'),
  ('body', 'black', 'light gray'),
  ('bg', 'black', 'light gray'),
  ('btn','light gray','dark cyan'),
  ('btnf','light gray','dark blue'),
  ('main_sel', 'black', 'light gray'),
  ('main_self', 'light gray', 'black'),
  ('selected', 'light gray', 'dark red'),
  ('error', 'light gray', 'dark red')
]

#Main function to start the program
def main():
  #creating the object to store all database information
  user_info = UserDBInfo()

  #This function handles all input that isn't directly handled by a widget. This is
  #what allows the user to exit the program.
  def unhandled_input(key):
    if key == 'q':
      if user_info.db_conn != "":
        try:
          user_info.db_conn.close()  
        except:
          var = 0
      raise urwid.ExitMainLoop()

  #call function to create login screen widget, which is opening widget used
  login_widget = loginview.create_main_view(user_info)
  
  #create loop variable and then call urwid event loop
  loop = urwid.MainLoop(login_widget, palette, unhandled_input=unhandled_input, screen=urwid.curses_display.Screen())
  loop.run()


if __name__ == '__main__':
  main()
